@@ -1,4 +1,5 @@
-# something something
+import pandas as pd
+import numpy as np 
 
 
 class Crp:
@@ -59,14 +60,56 @@ class Crp:
             else:
                 value = 0
             target.append([c_id,value])
-            
-            
+
         target = pd.DataFrame(target,columns = ['customerid','is_returned'])
         #Output
         return target
 
  #---------------------------------------------------------------------------------------------------------------------------
 
+
+
+    def getRecency(self,matrix,dateFlag):
+        """
+        Params:
+        
+            matrix -> data matrix
+            dateFlag    ->
+        ---------------------
+        Output:- entire modified dataframe with the recency feature
+        
+        """
+        matrix['Recency'] = matrix.apply(lambda x: dateFlag - pd.to_datetime(x.order_date),axis =1).dt.days
+        
+        output = matrix
+        
+        # Output
+        return output
+
+
+
+
+    def getLastOrderValue(self,matrix,train_df):
+        """
+        Params:
+            matrix      ->
+            train_df    ->
+        --------------------
+        Output:
+            output = matrix with last order value
+        """
+        
+        target = train_df[train_df.customerid.map(matrix['order_date']) == train_df.order_date][['customerid','totalprice']]
+        matrix = matrix.reset_index()
+        output = pd.merge(target,matrix,on='customerid')
+        output.drop_duplicates(inplace = True)
+        
+        # Output
+        return output
+
+
+
+
     def getFeaturesMatrix(train_df,org_df,timeFlag,items_dataset):
         """
         Params:
@@ -80,19 +123,59 @@ class Crp:
         
         # Get customer ids
         customers = train_df['customerid'].unique()
+        # Get the date which acts as reference point for future calculations
+        dateFlag_sep = np.max(org_df['order_date']) - pd.DateOffset(months=timeFlag)
+
+        grp = train_df.groupby('customerid')
+
+        grp_dict = {'order_date':'max','id':'count','totalprice':'sum'}
+
+        df_matrix = gb.agg(d).rename(columns={'id':'num_orders'})
+
+        df_matrix = self.getRecency(last_orders,dateFlag_sep)
+
+        df_matrix = self.getLastOrderValue(df_matrix,train_df)
+
+
+
+
         
-        Crp
+        output = df_matrix 
+        
+        return output
+
+
+
+# --------------------------------------------------------------------------------------------------------------------------------
+
+    def getMergedDF(self,data_list,key):
+        """
+        Params: 
+            data_list -> list that contains all dataframes
+            key - string, the mutual key for the merge. 
+        ---------------
+        Output: Merged dataset
+        
+        
+        """
+        output = reduce(lambda  left,right: pd.merge(left,right,on=[key],
+                                                how='outer'), data_list)
         
         
         return output
 
+
+# ------------------------------------------------------------------------------------------------------------------------------------
+
+
+
     def prepareDF(self,original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset):
         """
         Params:
             original_dataset ->
             timeFlag         ->
             train_window     ->
-        -------------------------------
+        -------------------------------__version__
         Output:
         
         """
@@ -100,25 +183,24 @@ class Crp:
         # First, perform initial cleaning and save a log file of actions
         # df,log_file = getCleanData(original_dataset)
         # Get the first matrices
-        train_data = self.getTargetframe(df,dateFlag_sep,date_flag_min)
+        train_data = self.getTrainingframe(df,dateFlag_sep,date_flag_min)
         print('train_data Done')
         # For the target values calculations first -> get the relevant dataframe with the relevant timeframe
-        target_frame = getTargetframe(df,dateFlag_sep,lag)
+        target_frame = self.getTargetframe(df,dateFlag_sep,lag)
         print('target frame done')
         # Calculate the target value
-        target_value = getTargetValues(train_data,target_frame)
+        target_value = self.getTargetValues(train_data,target_frame)
         print('target value done')
         # Calculate features
-        df = getFeaturesMatrix(train_data,orders,lag,items_dataset)
+        df = self.getFeaturesMatrix(train_data,orders,lag,items_dataset)
         
         # Merge feature sets
-        data = getMergedDF(df,'customerid')
-        
+        data = self.getMergedDF(df,'customerid')
         #Merge with target value 
         
         finalDF = pd.merge(data,target_value,on='customerid')
         
-        finalDf = finalDF.rename(columns = {'num_orders':'Num_orders','totalprice':'Total_spent'})
+        # finalDf = finalDF.rename(columns = {'num_orders':'Num_orders','totalprice':'Total_spent'})
         
         output =  finalDF
         
@@ -131,9 +213,7 @@ class Crp:
         Params:
             original_dataset - > the original raw data 
             lag              - > size ( in months) of the targetValue window
-            train_window     - > size ( in months ) of the Feature Matrix window 
-            
-        -------------------------------------------
+            train_window     - > size (__version__------------
         Output: Constructed dataframe 
         
         """
